import * as h from '@api-ts/io-ts-http';
declare type SuccessfulResponses<Route extends h.HttpRoute> = {
    [R in keyof Route['response']]: {
        status: R;
        error?: undefined;
        body: h.ResponseTypeForStatus<Route['response'], R>;
        original: Response;
    };
}[keyof Route['response']];
export declare type DecodedResponse<Route extends h.HttpRoute> = SuccessfulResponses<Route> | {
    status: 'decodeError';
    error: string;
    body: unknown;
    original: Response;
};
export declare class DecodeError extends Error {
    readonly decodedResponse: DecodedResponse<h.HttpRoute>;
    constructor(message: string, decodedResponse: DecodedResponse<h.HttpRoute>);
}
declare type ExpectedDecodedResponse<Route extends h.HttpRoute, StatusCode extends keyof Route['response']> = DecodedResponse<Route> & {
    status: StatusCode;
};
declare type PatchedRequest<Req, Route extends h.HttpRoute> = Req & {
    decode: () => Promise<DecodedResponse<Route>>;
    decodeExpecting: <StatusCode extends keyof Route['response']>(status: StatusCode) => Promise<ExpectedDecodedResponse<Route, StatusCode>>;
};
declare type SuperagentLike<Req> = {
    [K in h.Method]: (url: string) => Req;
};
export declare type Response = {
    body: unknown;
    status: number;
};
export interface SuperagentRequest<Res extends Response> extends Promise<Res> {
    ok(callback: (response: Res) => boolean): this;
    query(params: Record<string, string | string[]>): this;
    set(name: string, value: string): this;
    send(body: string): this;
}
export declare const substitutePathParams: (path: string, params: Record<string, string>) => string;
export declare type RequestFactory<Req> = <Route extends h.HttpRoute>(route: Route, params: Record<string, string>) => Req;
export declare const superagentRequestFactory: <Req>(superagent: SuperagentLike<Req>, base: string) => RequestFactory<Req>;
export declare const supertestRequestFactory: <Req>(supertest: SuperagentLike<Req>) => RequestFactory<Req>;
export declare type BoundRequestFactory<Req extends SuperagentRequest<Response>, Route extends h.HttpRoute> = (params: h.RequestType<Route>) => PatchedRequest<Req, Route>;
export declare const requestForRoute: <Req extends SuperagentRequest<Response>, Route extends h.HttpRoute<"get" | "post" | "put" | "delete" | "patch">>(requestFactory: RequestFactory<Req>, route: Route) => BoundRequestFactory<Req, Route>;
export {};
//# sourceMappingURL=request.d.ts.map