"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.requestForRoute = exports.supertestRequestFactory = exports.superagentRequestFactory = exports.substitutePathParams = exports.DecodeError = void 0;
const h = __importStar(require("@api-ts/io-ts-http"));
const E = __importStar(require("fp-ts/Either"));
const pipeable_1 = require("fp-ts/pipeable");
const PathReporter = __importStar(require("io-ts/lib/PathReporter"));
const node_path_1 = require("node:path");
const whatwg_url_1 = require("whatwg-url");
class DecodeError extends Error {
    constructor(message, decodedResponse) {
        super(message);
        this.decodedResponse = decodedResponse;
    }
}
exports.DecodeError = DecodeError;
const decodedResponse = (res) => res;
const substitutePathParams = (path, params) => {
    for (const key in params) {
        if (params.hasOwnProperty(key)) {
            path = path.replace(`{${key}}`, params[key]);
        }
    }
    return path;
};
exports.substitutePathParams = substitutePathParams;
const superagentRequestFactory = (superagent, base) => (route, params) => {
    const method = route.method.toLowerCase();
    if (!h.Method.is(method)) {
        throw Error(`Unsupported http method "${route.method}"`);
    }
    const url = new whatwg_url_1.URL(base);
    const substitutedPath = (0, exports.substitutePathParams)(route.path, params);
    url.pathname = node_path_1.posix.join(url.pathname, substitutedPath);
    return superagent[method](url.toString());
};
exports.superagentRequestFactory = superagentRequestFactory;
const supertestRequestFactory = (supertest) => (route, params) => {
    const method = route.method.toLowerCase();
    if (!h.Method.is(method)) {
        throw Error(`Unsupported http method "${route.method}"`);
    }
    const path = (0, exports.substitutePathParams)(route.path, params);
    return supertest[method](path);
};
exports.supertestRequestFactory = supertestRequestFactory;
const hasCodecForStatus = (responses, status) => {
    return status in responses && responses[status] !== undefined;
};
const stringify = (value) => JSON.stringify(value, (_key, value) => typeof value === 'bigint' ? value.toString() : value);
const patchRequest = (route, req) => {
    const patchedReq = req;
    patchedReq.decode = () => req.then((res) => {
        const { body, status } = res;
        if (!hasCodecForStatus(route.response, status)) {
            return decodedResponse({
                status: 'decodeError',
                error: `No codec for status ${status}`,
                body,
                original: res,
            });
        }
        return (0, pipeable_1.pipe)(route.response[status].decode(res.body), E.map((body) => decodedResponse({
            status,
            body,
            original: res,
        })), E.getOrElse((error) => decodedResponse({
            status: 'decodeError',
            error: PathReporter.failure(error).join('\n'),
            body: res.body,
            original: res,
        })));
    });
    patchedReq.decodeExpecting = (status) => patchedReq.decode().then((res) => {
        if (res.original.status !== status) {
            const error = `Unexpected response ${res.original.status}: ${stringify(res.original.body)}`;
            throw new DecodeError(error, res);
        }
        else if (res.status === 'decodeError') {
            const error = `Could not decode response ${res.original.status}: [${stringify(res.original.body)}] due to error [${res.error}]`;
            throw new DecodeError(error, res);
        }
        else {
            return res;
        }
    });
    patchedReq.ok(() => true);
    return patchedReq;
};
const requestForRoute = (requestFactory, route) => (params) => {
    var _a;
    const reqProps = route.request.encode(params);
    let path = route.path;
    for (const key in reqProps.params) {
        if (reqProps.params.hasOwnProperty(key)) {
            path = path.replace(`{${key}}`, reqProps.params[key]);
        }
    }
    let request = requestFactory(route, reqProps.params).query(reqProps.query);
    const headers = (_a = reqProps.headers) !== null && _a !== void 0 ? _a : {};
    for (const key in headers) {
        if (headers.hasOwnProperty(key)) {
            request = request.set(key, headers[key]);
        }
    }
    if (reqProps.body) {
        request.set('content-type', 'application/json');
        request = request.send(stringify(reqProps.body));
    }
    return patchRequest(route, request);
};
exports.requestForRoute = requestForRoute;
//# sourceMappingURL=request.js.map